@@ -1,30 +1,27 @@
-# Copyright (c) 2015-2016, 2018-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2019-2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Bryce Guinta <bryce.guinta@protonmail.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2021 David Liu <david@cs.toronto.edu>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
 """Various context related utilities, including inference and call contexts."""
+
+from __future__ import annotations
+
 import contextlib
 import pprint
-from typing import TYPE_CHECKING, List, MutableMapping, Optional, Sequence, Tuple
+from typing import TYPE_CHECKING, Dict, Optional, Sequence, Tuple
 
 if TYPE_CHECKING:
+    from astroid import constraint, nodes
     from astroid.nodes.node_classes import Keyword, NodeNG
 
+_InferenceCache = Dict[
+    Tuple["NodeNG", Optional[str], Optional[str], Optional[str]], Sequence["NodeNG"]
+]
 
-_INFERENCE_CACHE = {}
+_INFERENCE_CACHE: _InferenceCache = {}
 
 
-def _invalidate_cache():
+def _invalidate_cache() -> None:
     _INFERENCE_CACHE.clear()
 
 
@@ -41,16 +38,22 @@ class InferenceContext:
         "callcontext",
         "boundnode",
         "extra_context",
+        "constraints",
         "_nodes_inferred",
     )
 
     max_inferred = 100
 
-    def __init__(self, path=None, nodes_inferred=None):
+    def __init__(
+        self,
+        path=None,
+        nodes_inferred: list[int] | None = None,
+    ):
         if nodes_inferred is None:
             self._nodes_inferred = [0]
         else:
             self._nodes_inferred = nodes_inferred
+
         self.path = path or set()
         """
         :type: set(tuple(NodeNG, optional(str)))
@@ -59,22 +62,15 @@ class InferenceContext:
 
         Currently this key is ``(node, context.lookupname)``
         """
-        self.lookupname = None
-        """
-        :type: optional[str]
-
-        The original name of the node
+        self.lookupname: str | None = None
+        """The original name of the node.
 
         e.g.
         foo = 1
         The inference of 'foo' is nodes.Const(1) but the lookup name is 'foo'
         """
-        self.callcontext = None
-        """
-        :type: optional[CallContext]
-
-        The call arguments and keywords for the given context
-        """
+        self.callcontext: CallContext | None = None
+        """The call arguments and keywords for the given context."""
         self.boundnode = None
         """
         :type: optional[NodeNG]
@@ -91,10 +87,13 @@ class InferenceContext:
         for call arguments
         """
 
+        self.constraints: dict[str, dict[nodes.If, set[constraint.Constraint]]] = {}
+        """The constraints on nodes."""
+
     @property
-    def nodes_inferred(self):
+    def nodes_inferred(self) -> int:
         """
-        Number of nodes inferred in this context and all its clones/decendents
+        Number of nodes inferred in this context and all its clones/descendents
 
         Wrap inner value in a mutable cell to allow for mutating a class
         variable in the presence of __slots__
@@ -102,15 +101,11 @@ class InferenceContext:
         return self._nodes_inferred[0]
 
     @nodes_inferred.setter
-    def nodes_inferred(self, value):
+    def nodes_inferred(self, value: int) -> None:
         self._nodes_inferred[0] = value
 
     @property
-    def inferred(
-        self,
-    ) -> MutableMapping[
-        Tuple["NodeNG", Optional[str], Optional[str], Optional[str]], Sequence["NodeNG"]
-    ]:
+    def inferred(self) -> _InferenceCache:
         """
         Inferred node contexts to their mapped results
 
@@ -119,11 +114,10 @@ class InferenceContext:
         """
         return _INFERENCE_CACHE
 
-    def push(self, node):
+    def push(self, node) -> bool:
         """Push node into inference path
 
-        :return: True if node is already in context path else False
-        :rtype: bool
+        :return: Whether node is already in context path.
 
         Allows one to see if the given node has already
         been looked at for this inference context"""
@@ -134,7 +128,7 @@ class InferenceContext:
         self.path.add((node, name))
         return False
 
-    def clone(self):
+    def clone(self) -> InferenceContext:
         """Clone inference path
 
         For example, each side of a binary operation (BinOp)
@@ -145,6 +139,7 @@ class InferenceContext:
         clone.callcontext = self.callcontext
         clone.boundnode = self.boundnode
         clone.extra_context = self.extra_context
+        clone.constraints = self.constraints.copy()
         return clone
 
     @contextlib.contextmanager
@@ -153,10 +148,9 @@ class InferenceContext:
         yield
         self.path = path
 
-    def __str__(self):
+    def __str__(self) -> str:
         state = (
-            "%s=%s"
-            % (field, pprint.pformat(getattr(self, field), width=80 - len(field)))
+            f"{field}={pprint.pformat(getattr(self, field), width=80 - len(field))}"
             for field in self.__slots__
         )
         return "{}({})".format(type(self).__name__, ",\n    ".join(state))
@@ -169,20 +163,20 @@ class CallContext:
 
     def __init__(
         self,
-        args: List["NodeNG"],
-        keywords: Optional[List["Keyword"]] = None,
-        callee: Optional["NodeNG"] = None,
+        args: list[NodeNG],
+        keywords: list[Keyword] | None = None,
+        callee: NodeNG | None = None,
     ):
         self.args = args  # Call positional arguments
         if keywords:
-            keywords = [(arg.arg, arg.value) for arg in keywords]
+            arg_value_pairs = [(arg.arg, arg.value) for arg in keywords]
         else:
-            keywords = []
-        self.keywords = keywords  # Call keyword arguments
+            arg_value_pairs = []
+        self.keywords = arg_value_pairs  # Call keyword arguments
         self.callee = callee  # Function being called
 
 
-def copy_context(context: Optional[InferenceContext]) -> InferenceContext:
+def copy_context(context: InferenceContext | None) -> InferenceContext:
     """Clone a context if given, or return a fresh contexxt"""
     if context is not None:
         return context.clone()
@@ -190,7 +184,7 @@ def copy_context(context: Optional[InferenceContext]) -> InferenceContext:
     return InferenceContext()
 
 
-def bind_context_to_node(context, node):
+def bind_context_to_node(context: InferenceContext | None, node) -> InferenceContext:
     """Give a context a boundnode
     to retrieve the correct function name or attribute value
     with from further inference.
@@ -198,9 +192,6 @@ def bind_context_to_node(context, node):
     Do not use an existing context since the boundnode could then
     be incorrectly propagated higher up in the call stack.
 
-    :param context: Context to use
-    :type context: Optional(context)
-
     :param node: Node to do name lookups from
     :type node NodeNG:
 
