@@ -1,19 +1,19 @@
-# Copyright (c) 2015-2018 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Bryce Guinta <bryce.guinta@protonmail.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
 import importlib
+import sys
 import warnings
+from typing import Any
 
 import lazy_object_proxy
 
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal
+
 
 def lazy_descriptor(obj):
     class DescriptorProxy(lazy_object_proxy.Proxy):
@@ -23,7 +23,7 @@ def lazy_descriptor(obj):
     return DescriptorProxy(obj)
 
 
-def lazy_import(module_name):
+def lazy_import(module_name: str) -> lazy_object_proxy.Proxy:
     return lazy_object_proxy.Proxy(
         lambda: importlib.import_module("." + module_name, "astroid")
     )
@@ -33,12 +33,12 @@ def lazy_import(module_name):
 class Uninferable:
     """Special inference object, which is returned when inference fails."""
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "Uninferable"
 
     __str__ = __repr__
 
-    def __getattribute__(self, name):
+    def __getattribute__(self, name: str) -> Any:
         if name == "next":
             raise AttributeError("next method should not be called")
         if name.startswith("__") and name.endswith("__"):
@@ -50,7 +50,7 @@ class Uninferable:
     def __call__(self, *args, **kwargs):
         return self
 
-    def __bool__(self):
+    def __bool__(self) -> Literal[False]:
         return False
 
     __nonzero__ = __bool__
@@ -87,7 +87,7 @@ class BadUnaryOperationMessage(BadOperationMessage):
 
         return objtype
 
-    def __str__(self):
+    def __str__(self) -> str:
         if hasattr(self.operand, "name"):
             operand_type = self.operand.name
         else:
@@ -110,12 +110,12 @@ class BadBinaryOperationMessage(BadOperationMessage):
         self.right_type = right_type
         self.op = op
 
-    def __str__(self):
+    def __str__(self) -> str:
         msg = "unsupported operand type(s) for {}: {!r} and {!r}"
         return msg.format(self.op, self.left_type.name, self.right_type.name)
 
 
-def _instancecheck(cls, other):
+def _instancecheck(cls, other) -> bool:
     wrapped = cls.__wrapped__
     other_cls = other.__class__
     is_instance_of = wrapped is other_cls or issubclass(other_cls, wrapped)
@@ -139,3 +139,16 @@ def proxy_alias(alias_name, node_type):
         },
     )
     return proxy(lambda: node_type)
+
+
+def check_warnings_filter() -> bool:
+    """Return True if any other than the default DeprecationWarning filter is enabled.
+
+    https://docs.python.org/3/library/warnings.html#default-warning-filter
+    """
+    return any(
+        issubclass(DeprecationWarning, filter[2])
+        and filter[0] != "ignore"
+        and filter[3] != "__main__"
+        for filter in warnings.filters
+    )
